#include <stdio.h>
#include <stdlib.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include "math.h"
#include "formats/TGA.h"

int imageWidth;
int imageHeight;

struct SMapPos
{
    unsigned short int  x, y;
};

SMapPos* mapPos;

void initMapPos()
{
    mapPos = (SMapPos *)malloc(imageHeight * imageHeight * sizeof(SMapPos));

    float radius = (float)imageHeight / 2;

    for (int y = 0; y < imageHeight; y++)
        for (int x = 0; x < imageHeight; x++)
        {
            SMapPos* pos = &mapPos[y * imageHeight + x];

            float centeredX = (float)x - radius;
            float centeredY = (float)y - radius;

            float sineLatitude = centeredY / radius;
            float latitude = asin(sineLatitude);

            float circleRadius = radius * cos(latitude);
            float sineLongitude = centeredX / circleRadius;

            // inside circle ?
            if (sineLongitude >= -1.0f && sineLongitude <= 1.0f)
            {
                float longitude = asin(sineLongitude) + M_PI / 2;

                pos->x = (longitude * imageWidth / 2) / M_PI;
                pos->y = y;
            }
            else
            {
                pos->x = 0;
                pos->y = 0;
            }
        }
}

int main(int argc, char* argv[])
{
    // init the window
    Color* image = tga.load("earth.tga", &imageWidth, &imageHeight);

    gfx.init("Earth", imageHeight, imageHeight);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    // initialise the table
    initMapPos();
    int scroll = 0;

    while (sys.isQuitRequested() == false)
    {
        // draw the sphere
        for (int y = 0; y < imageHeight; ++y)
            for (int x = 0; x < imageHeight; ++x)
            {
                SMapPos* pos = &mapPos[y * imageHeight + x];
                int px = (pos->x + scroll) % imageWidth;
                int py =  pos->y;
                Color col = image[py * imageWidth + px];

                gfx.setPixel(x, y, col);
            }

        scroll--;
        if (scroll < 0)
            scroll += imageWidth;

        // render and handle events
        gfx.render();
        sys.processEvents();
        sys.wait(16);
    }

    gfx.quit();
    delete[] image;
    free(mapPos);

    return EXIT_SUCCESS;
}
